package com.fjhx.xmhjc.service.product.impl;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.fjhx.xmhjc.entity.product.po.ProductCategory;
import com.fjhx.xmhjc.entity.product.po.ProductSubCategory;
import com.fjhx.xmhjc.entity.product.vo.ProductSubCategoryVo;
import com.fjhx.xmhjc.mapper.product.ProductCategoryMapper;
import com.fjhx.xmhjc.service.product.ProductCategoryService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.xmhjc.service.product.ProductSubCategoryService;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.xmhjc.entity.product.vo.ProductCategoryVo;
import com.fjhx.xmhjc.entity.product.dto.ProductCategorySelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.fjhx.xmhjc.entity.product.dto.ProductCategoryDto;
import cn.hutool.core.bean.BeanUtil;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * <p>
 * 产品类目表 服务实现类
 * </p>
 *
 * @author hj
 * @since 2023-11-12
 */
@Service
public class ProductCategoryServiceImpl extends ServiceImpl<ProductCategoryMapper, ProductCategory> implements ProductCategoryService {
    @Resource
    private ProductSubCategoryService productSubCategoryService;

    @Override
    public List<ProductCategoryVo> getList(ProductCategorySelectDto dto) {
        IWrapper<ProductCategory> wrapper = getWrapper();
        wrapper.orderByDesc("pc", ProductCategory::getId);
        List<ProductCategoryVo> list = this.baseMapper.getList(wrapper);
        return list;
    }

    @Override
    public Page<ProductCategoryVo> getPage(ProductCategorySelectDto dto) {
        IWrapper<ProductCategory> wrapper = getWrapper();
        wrapper.keyword(dto, new SqlField("pc", ProductCategory::getName));
        if(StrUtil.isNotBlank(dto.getStatus())){
            wrapper.eq("pc", ProductCategory::getStatus, dto.getStatus());
        }
        wrapper.orderByAsc("pc", ProductCategory::getSort);
        Page<ProductCategoryVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public ProductCategoryVo detail(Long id) {
        ProductCategory ProductCategory = this.getById(id);
        ProductCategoryVo result = BeanUtil.toBean(ProductCategory, ProductCategoryVo.class);
        List<ProductSubCategory> subCategoryList = productSubCategoryService.listByParentId(id);
        List<ProductSubCategoryVo> subCategoryVos = BeanUtil.copyToList(subCategoryList, ProductSubCategoryVo.class);
        result.setSubCategoryList(subCategoryVos);
        return result;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void add(ProductCategoryDto productCategoryDto) {
        this.save(productCategoryDto);
        productSubCategoryService.dealSubCategory(productCategoryDto.getId(), productCategoryDto.getSubCategoryList());
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void edit(ProductCategoryDto productCategoryDto) {
        this.updateById(productCategoryDto);
        productSubCategoryService.dealSubCategory(productCategoryDto.getId(), productCategoryDto.getSubCategoryList());
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }


    @Override
    public List<ProductCategoryVo> listByOpen() {
        IWrapper<ProductCategory> wrapper = getWrapper();
        wrapper.orderByDesc("pc", ProductCategory::getId);
        wrapper.eq("pc", ProductCategory::getStatus, "1");
        List<ProductCategoryVo> list = this.baseMapper.getList(wrapper);
        return list;
    }
}
